package com.example.application.views.results;

import com.example.application.data.bean.CalculatedMatch;
import com.example.application.data.entity.Matchday;
import com.example.application.data.service.MatchService;
import com.example.application.data.service.MatchdayService;
import com.example.application.data.service.SeasonService;
import com.example.application.utils.StringUtils;
import com.example.application.utils.VaadinUtils;
import com.example.application.views.abstractnavigation.SeasonAndMatchdayNavigation;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.grid.ColumnTextAlign;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.grid.GridVariant;
import com.vaadin.flow.component.html.Label;
import com.vaadin.flow.component.orderedlayout.FlexComponent;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.function.ValueProvider;
import com.vaadin.flow.router.BeforeEvent;
import com.vaadin.flow.router.HasUrlParameter;
import com.vaadin.flow.router.WildcardParameter;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Optional;

public class MatchdayView extends VerticalLayout implements HasUrlParameter<String> {

    private final MatchService matchService;

    private final SeasonAndMatchdayNavigation seasonAndMatchdayNavigation;

    private final VerticalLayout matchdayLayout = new VerticalLayout();
    private final Label matchdayHeader = new Label();
    private final Grid<CalculatedMatch> grid = new Grid<>();

    private final Button prevButton;
    private final Button nextButton;

    public MatchdayView(@Autowired SeasonService seasonService, @Autowired MatchdayService matchdayService, @Autowired MatchService matchService) {
        this.seasonAndMatchdayNavigation = new SeasonAndMatchdayNavigation("results", seasonService, matchdayService);
        this.seasonAndMatchdayNavigation.setRunnableToBeRunAfterSelection(this::configureContent);
        this.matchService = matchService;

        prevButton = seasonAndMatchdayNavigation.getPrevButton();
        nextButton = seasonAndMatchdayNavigation.getNextButton();

        configureContentLayout();
    }

    ////////////
    // LAYOUT //
    ////////////


    protected void configureContentLayout() {
        setWidthFull();
        setHeightFull();
        setAlignItems(FlexComponent.Alignment.CENTER);
        add(seasonAndMatchdayNavigation.getSelectionLayout(), matchdayLayout);

        // TODO add background color for content
        matchdayHeader.addClassName("matchday_header"); // TODO: add dates
        matchdayHeader.setWidth("100%");
        HorizontalLayout matchdayHeaderLayout = new HorizontalLayout(prevButton, matchdayHeader, nextButton);
        matchdayHeaderLayout.setWidthFull();

        matchdayLayout.setPadding(false);
        matchdayLayout.add(matchdayHeaderLayout, grid);
        matchdayLayout.setAlignItems(FlexComponent.Alignment.CENTER);
        matchdayLayout.setWidth("");

        Label headerPlayer1 = new Label("Player 1");
        headerPlayer1.addClassName("column_header");
        Label headerPlayer2 = new Label("Player 2");
        headerPlayer2.addClassName("column_header");

        grid.addColumn(VaadinUtils.getPlayerRenderer(CalculatedMatch::getPlayer1))
                .setHeader(headerPlayer1)
                .setTextAlign(ColumnTextAlign.END)
                .setWidth("13em")
                .setFlexGrow(1);

        grid.addColumn((ValueProvider<CalculatedMatch, String>) this::getResultString)
                .setHeader("vs.")
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("6em");

        grid.addColumn(VaadinUtils.getPlayerRenderer(CalculatedMatch::getPlayer2))
                .setHeader(headerPlayer2)
                .setTextAlign(ColumnTextAlign.START)
                .setWidth("13em")
                .setFlexGrow(1);

        grid.setWidth("32em"); // TODO: find a way to set this dynamically based on column widths
        grid.setHeightByRows(true);

        grid.addThemeVariants(GridVariant.LUMO_NO_BORDER,
                GridVariant.LUMO_NO_ROW_BORDERS, GridVariant.LUMO_ROW_STRIPES);
    }

    private String getResultString(CalculatedMatch match) {
        return StringUtils.getResultString(match.getScore1(), match.getScore2());
    }


    /////////////
    // CONTENT //
    /////////////

    protected void configureContent() {
        Optional<Matchday> selectedMatchday = seasonAndMatchdayNavigation.getSelectedMatchday();
        selectedMatchday.ifPresent(matchday -> {
            matchdayHeader.setText(String.format("Matchday %s", matchday.toString()));
            grid.setItems(matchService.getCalculatedMatches(matchday));
        });
    }


    @Override
    public void setParameter(BeforeEvent event, @WildcardParameter String param) {
        String[] params = param.split("/");
        String seasonParam;
        String matchdayParam;
        switch (params.length) {
            case 0:
                seasonParam = "";
                matchdayParam = "";
                break;
            case 1:
                seasonParam = params[0];
                matchdayParam = "";
                break;
            default:
                seasonParam = params[0];
                matchdayParam = params[1];
        }
        seasonAndMatchdayNavigation.setParameter(seasonParam, matchdayParam);
    }
}
